SystemOrganization addCategory: #BattleShipTest!

TestCase subclass: #BattleShipTestCase
	instanceVariableNames: 'factory player'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BattleShipTest'!

!BattleShipTestCase methodsFor: 'tests' stamp: 'YuLin 5/4/2013 17:16'!
setUp
	factory := ShipFactory new.
	player := Player new gridHeight: 10 gridWidth: 10 name: 'Oscar' factory: factory.
	player addShip: 'Minesweeper' at: 5@2 rotated: false.
	player addShip: 'Minesweeper' at: 8@2 rotated: false.
	player addShip: 'Destroyer' at: 8@10 rotated: true.
	player addShip: 'Battleship' at: 2@1 rotated: false.
	! !

!BattleShipTestCase methodsFor: 'tests' stamp: 'YuLin 5/4/2013 18:16'!
testBigCaptainCellHit
	| cells set point |
	set := Set new.
	set add: 8 @ 9.
	set add: 4 @ 1.
	set do: [ :each | player hit: each ]. 	"hit once"
	cells := player peekGridToGui: nil.
	cells
		withIndicesDo: [ :cell :i :j | 
			point := Point x: i y: j.
			(set includes: point)
				ifTrue: [ self assert: cell = 'x' ]
				ifFalse: [ self assert: cell = '~' ] ].	
	set do: [ :each | player hit: each ]. "hit twice, and the ships are sunk"
	cells := player peekGridToGui: nil.
	set add: 8 @ 10.
	set add: 8 @ 8.
	set add: 2 @ 1.
	set add: 3 @ 1.
	set add: 5 @ 1.
	cells
		withIndicesDo: [ :cell :i :j | 
			point := Point x: i y: j.
			(set includes: point)
				ifTrue: [ self assert: cell = '@' ]
				ifFalse: [ self assert: cell = '~' ] ]! !

!BattleShipTestCase methodsFor: 'tests' stamp: 'YuLin 5/4/2013 20:53'!
testDestroy
	| set |
	set := Set new.
	set add: 5 @ 2.
	set add: 8 @ 2.
	set add: 8 @ 9.
	set add: 4 @ 1.
	
	set do: [ :each | player hit: each ].
	self assert: player grid nrOfavailableShips = 2.
	
	set do: [ :each | player hit: each ].
	self assert: player grid nrOfavailableShips = 0
	
	! !

!BattleShipTestCase methodsFor: 'tests' stamp: 'YuLin 5/4/2013 18:32'!
testGrid
	| grid ship1 ship2 set point cells |
	grid := Grid new height: 10 width: 10.
	ship1 := Ship named: 'Ship1' withLayout: 'ldr|ShipCell|BigCaptainCell|ShipCell'.
	ship2 := Ship named: 'Ship2' withLayout: 'url|ShipCell|ShipCell|BigCaptainCell'.
	ship2 rotate.
	grid at: 3 @ 8 addShip: ship1.
	grid at: 7 @ 3 addShip: ship2.
	set := Set new.
	set add: 3 @ 8.
	set add: 2 @ 8.
	set add: 2 @ 7.
	set add: 7 @ 3.
	set add: 8 @ 3.
	set add: 8 @ 2.
	cells := grid peekMyGridToGui.
	cells
		withIndicesDo: [ :cell :i :j | 
			point := Point x: i y: j.
			(set includes: point)
				ifTrue: [ self assert: cell = 'B' ]
				ifFalse: [ self assert: cell = '~' ] ]! !

!BattleShipTestCase methodsFor: 'tests' stamp: 'YuLin 5/4/2013 20:24'!
testMisses
	| cells set point |
	player hit: 1 @ 1.
	player hit: 10 @ 5.
	player hit: 3 @ 3.
	player hit: 7 @ 4.
	player hit: 2 @ 9.
	set := Set new.
	set add: 1 @ 1.
	set add: 10 @ 5.
	set add: 3 @ 3.
	set add: 7 @ 4.
	set add: 2 @ 9.
	cells := player peekGridToGui: nil.
	cells
		withIndicesDo: [ :cell :i :j | 
			point := Point x: i y: j.
			(set includes: point)
				ifTrue: [ self assert: cell = 'x' ]
				ifFalse: [ self assert: cell = '~' ] ]! !

!BattleShipTestCase methodsFor: 'tests' stamp: 'YuLin 5/4/2013 18:17'!
testShipCellHit
	| cells set point |
	set := Set new.
	set add: 6 @ 2.
	set add: 9 @ 2.
	set add: 8 @ 10.
	set add: 8 @ 8.
	set add: 2 @ 1.
	set add: 3 @ 1.
	set add: 5 @ 1.
	set do: [ :each | player hit: each ]. "hit once, and the ships are sink."
	cells := player peekGridToGui: nil.
	cells
		withIndicesDo: [ :cell :i :j | 
			point := Point x: i y: j.
			(set includes: point)
				ifTrue: [ self assert: cell = '@' ]
				ifFalse: [ self assert: cell = '~' ] ]! !

!BattleShipTestCase methodsFor: 'tests' stamp: 'YuLin 5/4/2013 17:53'!
testShipPlacement
	| cells set point |
	cells := player peekGridToGui: player.
	set := Set new.
	set add: 5 @ 2.
	set add: 6 @ 2.
	set add: 8 @ 2.
	set add: 9 @ 2.
	set add: 8 @ 10.
	set add: 8 @ 9.
	set add: 8 @ 8.
	set add: 2 @ 1.
	set add: 3 @ 1.
	set add: 4 @ 1.
	set add: 5 @ 1.
	cells
		withIndicesDo: [:cell :i :j  | 
			point := Point x: i y: j.
			(set includes: point)
				ifTrue: [ self assert: cell = 'B' ]
				ifFalse: [ self assert: cell = '~' ] ]! !

!BattleShipTestCase methodsFor: 'tests' stamp: 'YuLin 5/4/2013 18:11'!
testSmallCaptainCellHit
	| cells set point |
	set := Set new.
	set add: 5 @ 2.
	set add: 8 @ 2.
	set do: [ :each | player hit: each ].
	set add: 6 @ 2.
	set add: 9 @ 2.
	cells := player peekGridToGui: nil.
	cells
		withIndicesDo: [ :cell :i :j | 
			point := Point x: i y: j.
			(set includes: point)
				ifTrue: [ self assert: cell = '@' ]
				ifFalse: [ self assert: cell = '~' ] ]! !

TestCase subclass: #BigCaptainCellTest
	instanceVariableNames: 'cell shipName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BattleShipTest'!

!BigCaptainCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 19:09'!
setUp 
	shipName := 'Minesweeper'.
	cell := BigCaptainCell new ship: (ShipFactory new buildShip: shipName).! !

!BigCaptainCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 20:11'!
testHit
	|result |
	result := cell hit.
	
	self assert: result isSunk = false.
	self assert: result isHit = true.
	self assert: result asString = 'HIT'.
	self assert: cell isHit = true.
	self assert: cell getCharStatus = 'x'.
	self assert: cell isOccupied = true.
	
	result := cell hit.
	
	self assert: result isSunk = true.
	self assert: result affectedShipName = shipName.
	self assert: result asString = ('SUNK ', shipName).
	self assert: cell isHit = true.
	self assert: cell getCharStatus = '@'.
	self assert: cell isOccupied = true.! !

!BigCaptainCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 19:10'!
testNoHit	
	self assert: cell isHit = false.
	self assert: cell getCharStatus = '~'.
	self assert: cell isOccupied = true.! !

TestCase subclass: #LayoutTest
	instanceVariableNames: 'layoutPattern1 layoutPattern2 basePattern basePatternRotated baseLayout doubleLayout complexLayout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BattleShipTest'!

!LayoutTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 16:42'!
assertRotateLayout: aLayout withPattern: aPattern andInverted: anInvertedPattern
	aLayout rotate.
	self assert: aLayout  layout = anInvertedPattern  .

	aLayout rotate.
	self assert: aLayout  layout = aPattern  .! !

!LayoutTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 16:12'!
setUp
	"D->L
	L->U
	R->D
	U->R"

	basePattern := 'rldu'.
	basePatternRotated := 'dulr'.
	layoutPattern1 := basePattern, basePattern .
	layoutPattern2 := basePattern, '|extra1|extra2'.
	
	baseLayout := Layout withLayout: basePattern.
	self assert: baseLayout layout = basePattern.
	
	doubleLayout := Layout withLayout: layoutPattern1.
	self assert: doubleLayout layout = layoutPattern1.
	
	complexLayout := Layout withLayout: layoutPattern2 .
	self assert: complexLayout layout = basePattern.! !

!LayoutTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 16:44'!
testRotate
	self assertRotateLayout: baseLayout withPattern: basePattern andInverted: basePatternRotated .
	self assertRotateLayout: doubleLayout  withPattern: basePattern, basePattern  andInverted:  basePatternRotated, basePatternRotated.
	self assertRotateLayout: complexLayout withPattern: basePattern andInverted: basePatternRotated .! !

TestCase subclass: #ShipCellTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BattleShipTest'!

!ShipCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 18:56'!
testHit
	| cell result |
	cell := ShipCell  new.
	result := cell hit.
	
	self assert: result isHit = true.
	self assert: cell isHit = true.
	self assert: cell getCharStatus = '@'.
	self assert: cell isOccupied = true.! !

!ShipCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 18:54'!
testNoHit
	| cell |
	cell := ShipCell new.
	
	self assert: cell isHit = false.
	self assert: cell getCharStatus = '~'.
	self assert: cell isOccupied = true.! !

TestCase subclass: #ShipHitResultTest
	instanceVariableNames: 'shipName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BattleShipTest'!

!ShipHitResultTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/5/2013 17:23'!
setUp 
	shipName := 'Minesweeper'! !

!ShipHitResultTest methodsFor: 'tests' stamp: 'MihaiCodoban 5/4/2013 18:47'!
testHit
	| result |
	result := ShipHitResult new.
	result hit.
	self assert: result isHit = true.
	self assert: result isSunk = false.
	self assert: result isSurrender = false.
	self assert: result asString = 'HIT'.! !

!ShipHitResultTest methodsFor: 'tests' stamp: 'MihaiCodoban 5/5/2013 17:24'!
testHitandSink
	| result |
	
	result := ShipHitResult new.
	result hit.
	result sunk: shipName.
	
	self assert: result isHit = true.
	self assert: result isSunk = true.
	self assert: result isSurrender = false.
	self assert: result asString = ('SUNK ','Minesweeper').! !

!ShipHitResultTest methodsFor: 'tests' stamp: 'MihaiCodoban 5/4/2013 18:47'!
testHitandSinkandSurrender
	| result |
	
	result := ShipHitResult new.
	result hit.
	result sunk: (ShipFactory new buildShip: 'Minesweeper').
	result surrender.
	
	self assert: result isHit = true.
	self assert: result isSunk = true.
	self assert: result isSurrender = true.
	self assert: result asString = ('SURRENDER').! !

!ShipHitResultTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/5/2013 20:08'!
testMergeHitSunk
	self assert: (ShipHitResult new hit mergeWith: (ShipHitResult new sunk: shipName)) asString = ('SUNK ', shipName).
	self assert:  (ShipHitResult new sunk: shipName; mergeWith: (ShipHitResult new sunk: shipName)) asString = ('SUNK ',shipName , '|', shipName) .
	self assert:  (ShipHitResult new sunk: shipName; mergeWith: (ShipHitResult new surrender)) asString = 'SURRENDER' .! !

!ShipHitResultTest methodsFor: 'tests' stamp: 'MihaiCodoban 5/4/2013 18:47'!
testMiss
	| result |
	result := ShipHitResult new.
	
	self assert: result isHit = false.
	self assert: result isSunk = false.
	self assert: result isSurrender = false.
	self assert: result asString = 'MISS'! !

!ShipHitResultTest methodsFor: 'tests' stamp: 'MihaiCodoban 5/5/2013 17:23'!
testSink
	
	| result |
	result := ShipHitResult new.
	result sunk: shipName.
	self assert: result isHit = false.
	self assert: result isSunk = true.
	self assert: result isSurrender = false.
	self assert: result asString = ('SUNK', ' Minesweeper').! !

!ShipHitResultTest methodsFor: 'tests' stamp: 'MihaiCodoban 5/4/2013 18:47'!
testSinkAndSurrender
	| result |
	
	result := ShipHitResult new.
	result hit.
	result sunk: (ShipFactory new buildShip: 'Minesweeper').
	result surrender.
	
	self assert: result isHit = true.
	self assert: result isSunk = true.
	self assert: result isSurrender = true.
	self assert: result asString = ('SURRENDER').! !

!ShipHitResultTest methodsFor: 'tests' stamp: 'MihaiCodoban 5/5/2013 17:23'!
testSunk
	
	| result |
	result := ShipHitResult new.
	result sunk: shipName.
	self assert: result isHit = false.
	self assert: result isSunk = true.
	self assert: result isSurrender = false.
	self assert: result asString = ('SUNK', ' Minesweeper').! !

!ShipHitResultTest methodsFor: 'tests' stamp: 'MihaiCodoban 5/4/2013 18:47'!
testSurrender
	| result |
	
	result := ShipHitResult new.
	result surrender.
	
	self assert: result isHit = false.
	self assert: result isSunk = false.
	self assert: result isSurrender = true.
	self assert: result asString = ('SURRENDER').! !

TestCase subclass: #SmallCaptainCellTest
	instanceVariableNames: 'shipName cell'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BattleShipTest'!

!SmallCaptainCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 19:04'!
setUp 
	shipName := 'Minesweeper'.
	cell := SmallCaptainCell new ship: (ShipFactory new buildShip: shipName).! !

!SmallCaptainCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 19:08'!
testHit
	|result |
	result := cell hit.
	
	self assert: result isSunk = true.
	self assert: result affectedShipName = shipName.
	self assert: cell isHit = true.
	self assert: cell getCharStatus = '@'.
	self assert: cell isOccupied = true.! !

!SmallCaptainCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 19:10'!
testNoHit
	self assert: cell isHit = false.
	self assert: cell getCharStatus = '~'.
	self assert: cell isOccupied = true.! !

TestCase subclass: #WaterCellTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BattleShipTest'!

!WaterCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 18:47'!
testHit
	| cell result |
	cell := WaterCell new.
	result := cell hit.
	
	self assert: result isHit = false.
	self assert: cell isHit = true.
	self assert: cell getCharStatus = 'x'.
	self assert: cell isOccupied = false.! !

!WaterCellTest methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/4/2013 18:43'!
testNoHit
	| cell |
	cell := WaterCell new.
	
	self assert: cell isHit = false.
	self assert: cell getCharStatus = '~'.
	self assert: cell isOccupied = false.! !

TestCase subclass: #WeaponsTestCase
	instanceVariableNames: 'player1 factory player2 areaWeaponTrue areaWeaponOdd areaWeaponOnce'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BattleShipTest'!

!WeaponsTestCase methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/5/2013 19:41'!
setUp
	factory := ShipFactory new.
	player1 := Player new gridHeight: 10 gridWidth: 10 name: 'Oscar' factory: factory.
	player2 := Player new gridHeight: 10 gridWidth: 10 name: 'Bob' factory: factory.
	
	player2 addShip: 'Minesweeper' at: (3 @ 3) rotated: true.
	player2 addShip: 'Minesweeper' at: (6 @ 3) rotated: true.	
	player2 addShip: 'Minesweeper' at: (10 @ 3) rotated: true.
	
	areaWeaponTrue := 	WeaponsFactory new player: player1; makeStrikeWeaponWithActivationCondition: (TrueCondition new player: player1).
	areaWeaponOdd := 	WeaponsFactory new player: player1; makeStrikeWeaponWithActivationCondition: (OddSunkenShipsCondition new player: player1).
	areaWeaponOnce := 	WeaponsFactory new player: player1; makeStrikeWeaponWithActivationCondition: (OneShipSunkCondition new player: player1).! !

!WeaponsTestCase methodsFor: 'as yet unclassified' stamp: 'MihaiCodoban 5/7/2013 11:01'!
testUniqueHit
	| result |

	self assert: player2 nrOfAvailableShips = 3.
	self assert: player1 enemyShipsSunk  = 0.
	self assert: areaWeaponOnce canBeUsedByPlayer = false.
	self assert: player1 getUsableBonusWeapons  isEmpty .

	result := player1 attack: player2 At: (3@3) WithWeapon:  SimpleWeapon new.

	self assert: result asString = ('SUNK ', 'Minesweeper').	
	self assert: player2 nrOfAvailableShips = 2.
	self assert: player1 enemyShipsSunk  = 1.
	self assert: areaWeaponOnce canBeUsedByPlayer = true.		
	self assert: player1 getUsableBonusWeapons size = 1.

	result := player1 attack: player2 At: (6@3) WithWeapon: areaWeaponOnce .		
			
	self assert: result asString = ('SURRENDER').	
	self assert: player2 nrOfAvailableShips = 0.
	self assert: player1 enemyShipsSunk  = 3.
	self assert: areaWeaponOnce canBeUsedByPlayer = false.		
	self assert: player1 getUsableBonusWeapons isEmpty .	
	
	
! !
